package com.gic.haoban.manage.service.service.out.impl;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.wechat.api.service.qywx.QywxUserApiService;

import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.haoban.manage.api.dto.MemberUnionidRelatedDTO;
import com.gic.haoban.manage.api.service.DepartmentApiService;
import com.gic.haoban.manage.api.service.MemberUnionidRelatedApiService;
import com.gic.haoban.manage.api.service.WxEnterpriseApiService;
import com.gic.haoban.manage.service.entity.TabHaobanStaff;
import com.gic.haoban.manage.service.entity.TabHaobanWxEnterprise;
import com.gic.haoban.manage.service.entity.TabHaobanWxEnterpriseRelated;
import com.gic.haoban.manage.service.service.MemberUnionRelatedService;
import com.gic.haoban.manage.service.service.StaffService;
import com.gic.haoban.manage.service.service.WxEnterpriseRelatedService;
import com.gic.haoban.manage.service.service.WxEnterpriseService;
import com.gic.member.api.dto.MemberDTO;
import com.gic.member.api.service.MemberService;
import com.gic.redis.data.anno.RedisLock;
import com.gic.redis.data.util.RedisUtil;

@Service
public class MemberUnionidRelatedApiServiceImpl implements MemberUnionidRelatedApiService{
	private static final Logger log = LogManager.getLogger(MemberUnionidRelatedApiServiceImpl.class);
	@Autowired
	private MemberUnionRelatedService memberUnionRelatedService;
	@Autowired
	private StaffService staffService;
	@Autowired
	private QywxUserApiService qywxUserApiService;
	@Autowired
	private com.gic.enterprise.api.service.StoreGroupService gicDepartmentService;
	@Autowired
	private MemberService memberService;
	@Autowired
	private WxEnterpriseRelatedService wxEnterpriseRelatedService;
	@Autowired
	private WxEnterpriseService wxEnterpriseService;
	
	private static final String SELF_APP = "selfSuiteId";


	@Override
	public MemberUnionidRelatedDTO getByUnionId(String staffId, String unionid) {
		TabHaobanStaff staff = staffService.selectById(staffId);
		if(staff == null){
			return null;
		}
		log.info("【查询会员】wxUserId={},unionid={}",staff.getWxUserId(),unionid);
		return memberUnionRelatedService.getByUnionId(staff.getWxUserId(),unionid);
	}
	@Override
	public String addMemberUnionidRelated(MemberUnionidRelatedDTO dto) {
		log.info("【新增外部联系人回调】dto={}",JSON.toJSONString(dto));
		String corpId = dto.getCorpid();
		TabHaobanWxEnterprise wxEnterprise = wxEnterpriseService.getEnterpriseBycorpId(corpId);
		if(wxEnterprise == null){
			return "";
		}
		List<TabHaobanWxEnterpriseRelated> list = wxEnterpriseRelatedService.getByWxEnterpriseId(wxEnterprise.getWxEnterpriseId());
		String suiteid = dto.getSuiteid();		
		String unionIdJson = "";
		if("SELF_APP".equals(suiteid)){
			unionIdJson = qywxUserApiService.getCorpSelfExternalUseridInfo(dto.getCorpid(), wxEnterprise.getWxSecretKey(), dto.getExternalUserid());
		}else{
			unionIdJson = qywxUserApiService.getExternalUseridInfo(dto.getCorpid(), dto.getSuiteid(), dto.getExternalUserid());
		}
		
		log.info("【新增外部联系人回调】unionIdJson={}",JSON.toJSONString(unionIdJson));
		String unionId = "";
		String name = "";
		String follow_user = "";
		String createTime  = "";
		String userId = "";
		if(StringUtils.isNotBlank(unionIdJson)){
			JSONObject jo = JSON.parseObject(unionIdJson);
			unionId = jo.getString("unionid");
			suiteid = jo.getString("suiteid");
			name = jo.getString("name");
			follow_user = jo.getString("follow_user");
			if(StringUtils.isNotBlank(follow_user)){
				JSONObject j = JSON.parseObject(follow_user);
				createTime = j.getString("createtime");
				userId = j.getString("userid");
			}
		}
		
		String key = "external" + userId + "_" + name + "_" + createTime;
		RedisUtil.lock(key,2l);
		
		if("SELF_APP".equals(suiteid)){
			MemberDTO member = null;
			for (TabHaobanWxEnterpriseRelated tabHaobanWxEnterpriseRelated : list) {
				 member = memberService.getMemberByUnionid(unionId, tabHaobanWxEnterpriseRelated.getEnterpriseId());
				 if(member != null){
					 break;
				 }
			}
			if(member != null){
				dto.setMemberId(member.getMemberId());
			}
			dto.setSelfExternalUserid(dto.getExternalUserid());
			dto.setUnionid(unionId);
		}
		
		dto.setExternalName(name);
		dto.setAddCreateTime(createTime);
		if(StringUtils.isBlank(unionId)){
			return "";
		}
		
		dto.setWxEnterpriseId(wxEnterprise.getWxEnterpriseId());
		dto.setSuiteid(wxEnterprise.getWxSecretKey());
		
		MemberUnionidRelatedDTO exsitDTO = memberUnionRelatedService.getByParams(dto.getWxUserId(), wxEnterprise.getWxEnterpriseId(),name,createTime);
		String uuid = "";
		if(exsitDTO == null){
			uuid = memberUnionRelatedService.addMemberUnionidRelated(dto);
		}else{
			//如果是自建更新uuid
			if("SELF_APP".equals(suiteid)){
				exsitDTO.setUnionid(unionId);
				exsitDTO.setMemberId(dto.getMemberId());
			}else{
				//更新对外联系人id
				exsitDTO.setExternalUserid(dto.getExternalUserid());
			}
			memberUnionRelatedService.update(exsitDTO);
			uuid = exsitDTO.getMemberUnionidRelatedId();
		}
		return uuid;
	}
	@Override
	public void delMemberUnionidRelated(MemberUnionidRelatedDTO dto) {
		memberUnionRelatedService.delMemberUnionidRelated(dto);
		
	}

	private void getExternalUserUnionid(MemberUnionidRelatedDTO dto){
		TabHaobanWxEnterprise wxEnterprise = wxEnterpriseService.getEnterpriseBycorpId(dto.getCorpid());
		if(wxEnterprise == null){
			return ;
		}
		String externalUseridInfo = this.qywxUserApiService.getCorpSelfExternalUseridInfo(dto.getCorpid(), wxEnterprise.getWxSecretKey(), dto.getExternalUserid());
		if(StringUtils.isNotBlank(externalUseridInfo)){
			JSONObject jsonObject = JSON.parseObject(externalUseridInfo);
			dto.setUnionid(jsonObject.getString("unionid"));
		}
	}
	@Override
	public MemberUnionidRelatedDTO getByMemberId(String staffId, String memberId) {
		
		return null;
	}
	@Override
	public List<MemberUnionidRelatedDTO> getByWxUserIdAndWxEnterpriseId(String wxUserId, String wxEnterpriseId) {
		List<MemberUnionidRelatedDTO> list = memberUnionRelatedService.getByWxUserIdAndWxEnterpriseId(wxUserId,wxEnterpriseId);
		return list;
	}
}
